/**
 * GitlabRepository encapsulates all requiests to a single GitLab repository
 * 
 * @param {*} apiUrl url of API like https://eicweb.phy.anl.gov/api/v4
 * @param {*} projectId id of the project, like 473
 */
class GitlabRepository {
    constructor(apiUrl, projectId) {

        this.apiUrl = apiUrl;
        this.projectId = projectId;
        this.baseUrl = `${apiUrl}/projects/${projectId}`;
    }

    /**
     * Gets all branches for a project. (Actually the first page only)
     *
     * @returns branches object future
     */
    async getBranches() {
        console.log(`GitlabRepository ${this.projectId}: Loading repository branches...`);
        // The requiest is like this
        // https://eicweb.phy.anl.gov/api/v4/projects/473/repository/branches?per_page=100
        let response = await fetch(`${this.baseUrl}/repository/branches?per_page=100`);
        return await response.json();
    }

    /**
     * Gets pipelines for a branch. Sorted by finish time (recent to old)
     *
     * @param {str} branch branch name
     * @returns pipelines array
     */
    async getPipelinesForBranch(branch) {
        console.log(`GitlabRepository ${this.projectId}: Loading pipelines for a branch ${branch}...`);
        // The request is like: 
        // https://eicweb.phy.anl.gov/api/v4/projects/473/pipelines?ref=${branch}
        let pipelinesRequest = await fetch(`${this.baseUrl}/pipelines?ref=${branch}`);
        return await pipelinesRequest.json();
    }

    /**
     * 
     * @returns 
     */
    async getLatestPipelineForBranch(branch) {

        // Loading pipelines for a branch
        let pipelinesRequest = await this.getPipelinesForBranch(branch);
        
        if (pipelinesRequest.length == 0) {
            console.log(`GitlabRepository ${this.projectId}: No pipelines found for the ${branch} branch`);     // TODO throw error here
            return null;
        };

        return pipelinesRequest[0]
    }

    /**
     * Gets pipelines for a project. Sorted by finish time (recent to old)
     *
     * @returns pipelines array
     */
    async getPipelines() {
        console.log(`GitlabRepository ${this.projectId}: Loading pipelines`);
        // The request is like: 
        // https://eicweb.phy.anl.gov/api/v4/projects/473/pipelines
        let pipelinesRequest = await fetch(`${this.baseUrl}/pipelines?per_page=100`);
        if(pipelinesRequest.status != 200) {
            return null;
        }
        return await pipelinesRequest.json();
    }

    /**
     * Gets all branches for a project. (Actually the first page only)
     *
     * @returns branches object future
     */
    async getPipelineById(pipelineId) {
        console.log(`GitlabRepository ${this.projectId}: Loading pipeline by ID: ${pipelineId}`);
        // The request is like: 
        // https://eicweb.phy.anl.gov/api/v4/projects/473/pipelines/3453463
        let pipelinesRequest = await fetch(`${this.baseUrl}/pipelines/${pipelineId}`);
        
        if(pipelinesRequest.status != 200) {
            return null;
        }
        return await pipelinesRequest.json();
    }

    /**
     * Returns the first page of jobs for a given pipeline ID
     *
     * @param {int} pipelineId Id of the pipeline to get jobs
     * @returns
     */
    async getJobs(pipelineId) {
        console.log(`GitlabRepository ${this.projectId}: Loading jobs for a pipeline ${pipelineId}...`);
        let jobsRequest = await fetch(`${this.baseUrl}/pipelines/${pipelineId}/jobs?per_page=100`);
        if(jobsRequest.status != 200) {
            return null;
        }
        return await jobsRequest.json();
    }

    /**
     * Returns the job by pipelineId and name
     *
     * @param {int} pipelineId Id of the pipeline to get jobs
     * @param {str} name name of the job
     * @returns
     */
    async getJob(pipelineId, name) {

        console.log(`GitlabRepository ${this.projectId}: Looking for a job name ${name} in pipeline: ${pipelineId}...`);

        let jobs = await this.getJobs(pipelineId);
        for(let job of jobs) {
            if (job.name == "report") {
                return job;
            }
        }

        return null;
    }
}



export default GitlabRepository;